=== modified file 'HACKING'
--- HACKING	2006-11-03 01:52:12 +0000
+++ HACKING	2007-01-09 15:19:18 +0000
@@ -209,6 +209,9 @@
 
 Consider naming classes as nouns and functions/methods as verbs.
 
+Try to avoid using abbreviations in names, because there can be
+inconsistency if other people use the full name.
+
 
 Standard names
 --------------

=== modified file 'bzrlib/builtins.py'
--- bzrlib/builtins.py	2007-01-17 14:33:15 +0000
+++ bzrlib/builtins.py	2007-01-17 18:41:07 +0000
@@ -51,7 +51,7 @@
 """)
 
 from bzrlib.commands import Command, display_command
-from bzrlib.option import Option
+from bzrlib.option import Option, RegistryOption
 from bzrlib.progress import DummyProgress, ProgressPhase
 from bzrlib.trace import mutter, note, log_error, warning, is_quiet, info
 
@@ -1079,12 +1079,14 @@
     """
     takes_args = ['location?']
     takes_options = [
-                     Option('format', 
+                     RegistryOption('format',
                             help='Specify a format for this branch. Current'
                                  ' formats are: default, knit, metaweave and'
                                  ' weave. Default is knit; metaweave and'
                                  ' weave are deprecated',
-                            type=get_format_type),
+                            registry=bzrdir.format_registry,
+                            converter=get_format_type,
+                            value_switches=True),
                      ]
     def run(self, location=None, format=None):
         if format is None:
@@ -1137,12 +1139,14 @@
         (add files here)
     """
     takes_args = ["location"] 
-    takes_options = [Option('format', 
+    takes_options = [RegistryOption('format',
                             help='Specify a format for this repository.'
                                  ' Current formats are: default, knit,'
                                  ' metaweave and weave. Default is knit;'
                                  ' metaweave and weave are deprecated',
-                            type=get_format_type),
+                            registry=bzrdir.format_registry,
+                            converter=get_format_type,
+                            value_switches=True),
                      Option('trees',
                              help='Allows branches in repository to have'
                              ' a working tree')]
@@ -1967,12 +1971,14 @@
     """
     takes_args = ['url?']
     takes_options = [
-                     Option('format', 
-                            help='Upgrade to a specific format. Current formats'
-                                 ' are: default, knit, metaweave and weave.'
-                                 ' Default is knit; metaweave and weave are'
-                                 ' deprecated',
-                            type=get_format_type),
+                    RegistryOption('format',
+                        help='Upgrade to a specific format. Current formats'
+                             ' are: default, knit, metaweave and weave.'
+                             ' Default is knit; metaweave and weave are'
+                             ' deprecated',
+                        registry=bzrdir.format_registry,
+                        converter=get_format_type,
+                        value_switches=True),
                     ]
 
 

=== modified file 'bzrlib/errors.py'
--- bzrlib/errors.py	2007-01-17 15:37:08 +0000
+++ bzrlib/errors.py	2007-01-17 18:41:07 +0000
@@ -294,6 +294,14 @@
 
     _fmt = "Error in command line options"
 
+
+class BadOptionValue(BzrError):
+
+    _fmt = """Bad value "%(value)s" for option "%(name)s"."""
+
+    def __init__(self, name, value):
+        BzrError.__init__(self, name=name, value=value)
+
     
 class StrictCommitFailed(BzrError):
 

=== modified file 'bzrlib/option.py'
--- bzrlib/option.py	2007-01-12 03:52:12 +0000
+++ bzrlib/option.py	2007-01-17 16:32:40 +0000
@@ -203,6 +203,75 @@
         yield self.name, self.short_name(), argname, self.help
 
 
+class RegistryOption(Option):
+    """Option based on a registry
+
+    The values for the options correspond to entries in the registry.  Input
+    must be a registry key.  After validation, it is converted into an object
+    using Registry.get or a caller-provided converter.
+    """
+
+    def validate_value(self, value):
+        """Validate a value name"""
+        if value not in self.registry:
+            raise errors.BadOptionValue(self.name, value)
+
+    def convert(self, value):
+        """Convert a value name into an output type"""
+        self.validate_value(value)
+        if self.converter is None:
+            return self.registry.get(value)
+        else:
+            return self.converter(value)
+
+    def __init__(self, name, help, registry, converter=None,
+        value_switches=False):
+        """
+        Constructor.
+
+        :param name: The option name.
+        :param help: Help for the option.
+        :param registry: A Registry containing the values
+        :param converter: Callable to invoke with the value name to produce
+            the value.  If not supplied, self.registry.get is used.
+        :param value_switches: If true, each possible value is assigned its
+            own switch.  For example, instead of '--format metaweave',
+            '--metaweave' can be used interchangeably.
+        """
+        Option.__init__(self, name, help, type=self.convert)
+        self.registry = registry
+        self.name = name
+        self.converter = converter
+        self.value_switches = value_switches
+
+    def add_option(self, parser, short_name):
+        """Add this option to an Optparse parser"""
+        Option.add_option(self, parser, short_name)
+        if self.value_switches:
+            for key in self.registry.keys():
+                option_strings = ['--%s' % key]
+                parser.add_option(action='callback',
+                              callback=self._optparse_value_callback(key),
+                                  help=self.registry.get_help(key),
+                                  *option_strings)
+
+    def _optparse_value_callback(self, cb_value):
+        def cb(option, opt, value, parser):
+            setattr(parser.values, self.name, self.type(cb_value))
+        return cb
+
+    def iter_switches(self):
+        """Iterate through the list of switches provided by the option
+
+        :return: an iterator of (name, short_name, argname, help)
+        """
+        for value in Option.iter_switches(self):
+            yield value
+        if self.value_switches:
+            for key in sorted(self.registry.keys()):
+                yield key, None, None, self.registry.get_help(key)
+
+
 class OptionParser(optparse.OptionParser):
     """OptionParser that raises exceptions instead of exiting"""
 

=== modified file 'bzrlib/tests/test_options.py'
--- bzrlib/tests/test_options.py	2007-01-12 03:52:12 +0000
+++ bzrlib/tests/test_options.py	2007-01-17 16:32:40 +0000
@@ -14,13 +14,16 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
+from bzrlib import (
+    builtins,
+    bzrdir,
+    errors,
+    option,
+    repository,
+    symbol_versioning,
+    )
 from bzrlib.builtins import cmd_commit, cmd_log, cmd_status
 from bzrlib.commands import Command, parse_args
-from bzrlib import (
-        errors,
-        option,
-        symbol_versioning,
-        )
 from bzrlib.tests import TestCase
 
 def parse(options, args):
@@ -110,21 +113,69 @@
         """Test that we can pass a plain '-' as an argument."""
         self.assertEqual((['-'], {}), parse_args(cmd_commit(), ['-']))
 
+    def parse(self, options, args):
+        parser = option.get_optparser(dict((o.name, o) for o in options))
+        return parser.parse_args(args)
+
     def test_conversion(self):
         options = [option.Option('hello')]
-        opts, args = parse(options, ['--no-hello', '--hello'])
+        opts, args = self.parse(options, ['--no-hello', '--hello'])
         self.assertEqual(True, opts.hello)
-        opts, args = parse(options, [])
+        opts, args = self.parse(options, [])
         self.assertEqual(option.OptionParser.DEFAULT_VALUE, opts.hello)
-        opts, args = parse(options, ['--hello', '--no-hello'])
+        opts, args = self.parse(options, ['--hello', '--no-hello'])
         self.assertEqual(False, opts.hello)
         options = [option.Option('number', type=int)]
-        opts, args = parse(options, ['--number', '6'])
+        opts, args = self.parse(options, ['--number', '6'])
         self.assertEqual(6, opts.number)
-        self.assertRaises(errors.BzrCommandError, parse, options, ['--number'])
-        self.assertRaises(errors.BzrCommandError, parse, options, 
+        self.assertRaises(errors.BzrCommandError, self.parse, options,
+                          ['--number'])
+        self.assertRaises(errors.BzrCommandError, self.parse, options,
                           ['--no-number'])
 
+    def test_registry_conversion(self):
+        registry = bzrdir.BzrDirFormatRegistry()
+        registry.register_metadir('one', 'RepositoryFormat7', 'one help')
+        registry.register_metadir('two', 'RepositoryFormatKnit1', 'two help')
+        registry.set_default('one')
+        options = [option.RegistryOption('format', '', registry, str)]
+        opts, args = self.parse(options, ['--format', 'one'])
+        self.assertEqual({'format':'one'}, opts)
+        opts, args = self.parse(options, ['--format', 'two'])
+        self.assertEqual({'format':'two'}, opts)
+        self.assertRaises(errors.BadOptionValue, self.parse, options,
+                          ['--format', 'three'])
+        self.assertRaises(errors.BzrCommandError, self.parse, options,
+                          ['--two'])
+        options = [option.RegistryOption('format', '', registry, str,
+                   value_switches=True)]
+        opts, args = self.parse(options, ['--two'])
+        self.assertEqual({'format':'two'}, opts)
+        opts, args = self.parse(options, ['--two', '--one'])
+        self.assertEqual({'format':'one'}, opts)
+        opts, args = self.parse(options, ['--two', '--one',
+                                          '--format', 'two'])
+        self.assertEqual({'format':'two'}, opts)
+
+    def test_registry_converter(self):
+        options = [option.RegistryOption('format', '',
+                   bzrdir.format_registry, builtins.get_format_type)]
+        opts, args = self.parse(options, ['--format', 'knit'])
+        self.assertIsInstance(opts.format.repository_format,
+                              repository.RepositoryFormatKnit1)
+
+    def test_help(self):
+        registry = bzrdir.BzrDirFormatRegistry()
+        registry.register_metadir('one', 'RepositoryFormat7', 'one help')
+        registry.register_metadir('two', 'RepositoryFormatKnit1', 'two help')
+        registry.set_default('one')
+        options = [option.RegistryOption('format', 'format help', registry,
+                   str, value_switches=True)]
+        parser = option.get_optparser(dict((o.name, o) for o in options))
+        value = parser.format_option_help()
+        self.assertContainsRe(value, 'format.*format help')
+        self.assertContainsRe(value, 'one.*one help')
+
     def test_iter_switches(self):
         opt = option.Option('hello', help='fg')
         self.assertEqual(list(opt.iter_switches()),
@@ -135,6 +186,22 @@
         opt = option.Option('hello', help='fg', type=int, argname='gar')
         self.assertEqual(list(opt.iter_switches()),
                          [('hello', None, 'GAR', 'fg')])
+        registry = bzrdir.BzrDirFormatRegistry()
+        registry.register_metadir('one', 'RepositoryFormat7', 'one help')
+        registry.register_metadir('two', 'RepositoryFormatKnit1', 'two help')
+        registry.set_default('one')
+        opt = option.RegistryOption('format', 'format help', registry,
+                                    value_switches=False)
+        self.assertEqual(list(opt.iter_switches()),
+                         [('format', None, 'ARG', 'format help')])
+        opt = option.RegistryOption('format', 'format help', registry,
+                                    value_switches=True)
+        self.assertEqual(list(opt.iter_switches()),
+                         [('format', None, 'ARG', 'format help'),
+                          ('default', None, None, 'one help'),
+                          ('one', None, None, 'one help'),
+                          ('two', None, None, 'two help'),
+                          ])
 
 #     >>> parse_args('log -r 500'.split())
 #     (['log'], {'revision': [<RevisionSpec_int 500>]})

